#!/bin/bash
#
# Tool for report VZAgent related info & statistic.
#
# ToDo:
#	* process mem & cpu usage report
#	* DB size, table counts report  
#

# VZL defines
. /var/vzagent/etc/vzl_defines

# VZL functions.
. /var/vzagent/etc/vzl_functions

YES="Yes"
NO="!!NO!!"

function getInfo()
{
	local msg=$1
	shift
	echo "---- $msg ----"
	eval $*
	echo ""
}

function reportNodeInfo()
{
	hostname -f
	uname -a
	cat /proc/meminfo | grep MemTotal
	cat /proc/cpuinfo | grep 'processor\|model name\|cpu MHz'
	ip addr show
}

function getXmlFromAgentConfig()
{
	local seek=$1
	shift
	${VZLCORE_BIN_DIR}/vzlxmlman -g -m "$seek" $* ${VZLCORE_ETC_DIR}/vzagent.conf
}

function reportVeidToEidMap()
{
	local veid=""
	for i in `ls ${VZLCORE_ETC_DIR}/configs/`; do
		veid=`${VZLCORE_BIN_DIR}/vzlxmlman -g "config/veid" ${VZLCORE_ETC_DIR}/configs/$i`
		[ -n "$veid" ] || continue 
		printf "%-6s   %s\n" $veid $i
	done
}

function reportLdapInfo()
{
	vzctl exec 1 /etc/init.d/ldap status
}

function reportDbInfo()
{
	return 0
}

function reportProcessStateInfo()
{
	ps w --forest -o pid,stat,%cpu,cputime,%mem,rss:8,vsize:8,etime,cmd -C vzagent
}

function pingCheck()
{
	local addr=$1
	ping -w 2 -q $addr > /dev/null
}

function agentAccessCheck()
{
	local addr=$1
	/opt/vzagent/bin/sendRequestViaTCP.pl $addr >/dev/null 2>&1
}

function reportClusterInfo()
{
	local pFormat=" %-38s%-13s%-7s%-7s\n"
	local slavePFormat=" %-38s%-13s%-7s%-7s%-7s\n"
	local i=""
	local addr=""
	local ldap=""
	local connect=""
	local ping=""

	echo "Node ID: " `getXmlFromAgentConfig "" "data/system/configuration/id"`
	i=`getXmlFromAgentConfig "" "data/server_group/configuration/master/id"`
	echo "Master: "
	{
		local _addr=""
		printf "$pFormat" "EID" "Address" "Agent" "Ping"
		_addr=`getXmlFromAgentConfig "data/system/configuration/map/node[id=$i]/" "conn_info/address"`
		addr=$_addr
		[ "$addr" = "local" ] && addr="127.0.0.1"
		ping=$NO
		pingCheck $addr && ping=$YES
		connect=$NO
		agentAccessCheck $addr && connect=$YES
		printf "$pFormat" $i $_addr $connect $ping	
	}

	echo "Server group realm: " `getXmlFromAgentConfig "" "data/server_group/configuration/realm"`
	echo "Server group nodes:"
	printf "$slavePFormat" "EID" "Address" "Agent" "Ping" "Ldap"
	for i in `getXmlFromAgentConfig "" "data/server_group/configuration/slaves/node[*]/id"`; do
		ldap=$NO
		{ vzctl exec 1 slapcat -n 2 2>/dev/null | grep -q cn=$i,cn=Agent 2>/dev/null; } && \
			ldap=$YES
		addr=`getXmlFromAgentConfig "data/system/configuration/map/node[id=$i]/" "conn_info/address"`
		ping=$NO
		pingCheck $addr && ping=$YES
		connect=$NO
		agentAccessCheck $addr && connect=$YES
		printf "$slavePFormat" $i $addr $connect $ping $ldap
	done
}

function reportDumps()
{
	echo "Total size: " `du -sh /var/vzagent/dumps/`
	ls -la /var/vzagent/dumps/ | tail -n 10
}

function reportSOAPInfo()
{
    vzctl exec 1 netstat -nlp|grep 8080
    vzctl exec 1 netstat -nlp|grep 4646
}

function reportVznetCfgHN()
{
    vznetcfg if list
}

function reportPerfmonInfo()
{
	echo "Total size: " `du -sh /var/vzagent/tmp/perfdata/`
	local count=`ls -1 /var/vzagent/tmp/perfdata | wc -l`
	echo "Files count: " $count
}

function reportPstorageInfo()
{
	ls -la /var/vzagent/.pstorage/
}

# HN vz rpm versions
getInfo "Node info" "reportNodeInfo"

# HN vz rpm versions
getInfo "HN vz rpm versions" "rpm -qa | grep vzagent && rpm -q virtuozzo-release virtuozzo-tools vzsvetools vzctl vzmigrate vznetcfg vztt"

# SVE vz rpm versions
getInfo "SVE vz rpm versions" "vzctl exec 1 rpm -qa | grep vz"

# List of VEs
getInfo "List of VEs" "vzlist -a"

# Veid -> Eid map
getInfo "Veid -> Eid map" "reportVeidToEidMap | sort -n"

# VZAgent process info
getInfo "Process state info" "reportProcessStateInfo"

# VZAgent dumps report
getInfo "Dumps info" "reportDumps"

# Ldap info
getInfo "Ldap info" "reportLdapInfo"

# VZAgent cluster info report
getInfo "Cluster info" "reportClusterInfo"

# SOAP connectivity
getInfo "SOAP info" "reportSOAPInfo"

# VZNetConfig
getInfo "Vznet config" "reportVznetCfgHN"

# Perfmon data
getInfo "VZAgent perfmon info" "reportPerfmonInfo"

# VZAgent pstorage list
getInfo "VZAgent pstorage list" "reportPstorageInfo"
